@@ -1,77 +1,74 @@
 from tuprolog import logger
-
-# noinspection PyUnresolvedReferences
 import jpype
-# noinspection PyUnresolvedReferences
 import jpype.imports
-
 from ._ktadapt import *
-
 # noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Atom
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Block
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Clause
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Cons
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Constant
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Directive
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Empty
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import EmptyBlock
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import EmptyList
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Fact
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Formatter
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Indicator
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Integer
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import List
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Numeric
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Real
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Recursive
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Rule
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Scope
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Struct
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Substitution
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Term
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import TermComparator
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import TermConvertible
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import TermFormatter
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Terms
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import TermVisitor
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Truth
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Tuple
-# noinspection PyUnresolvedReferences
-from it.unibo.tuprolog.core import Var
-
+import it.unibo.tuprolog.core as _core
 from tuprolog.pyutils import iterable_or_varargs
 from tuprolog.jvmutils import jiterable, jmap
-
 from typing import Iterable, Union, Dict
+from ._ktmath import *
+
+
+Atom = _core.Atom
+
+Block = _core.Block
+
+Clause = _core.Clause
+
+Cons = _core.Cons
+
+Constant = _core.Constant
+
+Directive = _core.Directive
+
+Empty = _core.Empty
+
+EmptyBlock = _core.EmptyBlock
+
+EmptyList = _core.EmptyList
+
+Fact = _core.Fact
+
+Formatter = _core.Formatter
+
+Indicator = _core.Indicator
+
+Integer = _core.Integer
+
+List = _core.List
+
+Numeric = _core.Numeric
+
+Real = _core.Real
+
+Recursive = _core.Recursive
+
+Rule = _core.Rule
+
+Scope = _core.Scope
+
+Struct = _core.Struct
+
+Substitution = _core.Substitution
+
+Term = _core.Term
+
+TermComparator = _core.TermComparator
+
+TermConvertible = _core.TermConvertible
+
+TermFormatter = _core.TermFormatter
+
+Terms = _core.Terms
+
+TermVisitor = _core.TermVisitor
+
+Truth = _core.Truth
+
+Tuple = _core.Tuple
+
+Var = _core.Var
 
 
 @jpype.JImplements(TermConvertible)
@@ -120,18 +117,38 @@ def indicator(name: Union[str, Term], arity: Union[int, Term]) -> Indicator:
     return Indicator.of(name, arity)
 
 
-def integer(value: Union[int, str]) -> Integer:
+def numeric(value: Union[int, BigInteger, BigDecimal, str, float]) -> Numeric:
+    if isinstance(value, str):
+        return Numeric.of(jpype.JString @ value)
+    if isinstance(value, BigInteger):
+        return Integer.of(BigInteger @ value)
+    if isinstance(value, BigDecimal):
+        return Real.of(BigDecimal @ value)
+    if isinstance(value, int):
+        return Integer.of(jpype.JLong @ value)
+    if isinstance(value, float):
+        return Real.of(jpype.JDouble @ value)
+    return Numeric.of(value)
+
+
+def integer(value: Union[int, BigInteger, str]) -> Integer:
     if isinstance(value, str):
         return Integer.of(jpype.JString @ value)
-    else:
+    if isinstance(value, BigInteger):
+        return Integer.of(BigInteger @ value)
+    if isinstance(value, int):
         return Integer.of(jpype.JLong @ value)
+    return Integer.of(value)
 
 
-def real(value: Union[float, str]) -> Real:
+def real(value: Union[float, BigDecimal, str]) -> Real:
     if isinstance(value, str):
         return Real.of(jpype.JString @ value)
-    else:
+    if isinstance(value, BigDecimal):
+        return Real.of(BigDecimal @ value)
+    if isinstance(value, float):
         return Real.of(jpype.JDouble @ value)
+    return Real.of(value)
 
 
 def rule(head: Struct, *body: Union[Term, Iterable[Term]]) -> Rule:
@@ -179,4 +196,9 @@ def failed() -> Substitution.Fail:
     return Substitution.failed()
 
 
+EMPTY_UNIFIER: Substitution.Unifier = substitution()
+
+FAILED_SUBSTITUTION: Substitution.Fail = failed()
+
+
 logger.debug("Loaded JVM classes from it.unibo.tuprolog.core.*")
